unit Demo;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, RKcomp, StdCtrls, Quickrep, Buttons, Printers;

type
  TForm1 = class(TForm)
    Button1: TButton;
    Button2: TButton;
    Button3: TButton;
    BitBtn1: TBitBtn;
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.Button1Click(Sender: TObject);
var
   R : TRKPrinter;
begin
  Try
   R := TRKPrinter.Create;
   with R do begin
       R.LoadFromFile('about.rpt');
       Preview;
   end;
  finally
   R.free;
  end;
end;

procedure TForm1.Button2Click(Sender: TObject);
var
   R : TRKPrinter;
   h,w: Integer;
   b: TBitmap;
   s: string;
   tabs: array[1..4] of integer;
   myRect: TRect;
   oldcolor: TColor;
const
   inch = 1440;
   half = 720;
begin
(* In this demo I want to show how we can take full advantage of the TCanvas
   methods provided in Delphi and some important Windows API functions for
   Text Output. It shows how we can mix Delphi Methods with Windows API.
   For further reading I strongly recommend "Programming Windows 95"
   by Charles Petzold.
*)
   R := TRKPrinter.Create;
   with R do begin
       Margins := Rect(inch,inch,inch,inch); {Left,Top,Right,Bottom
       one thing should be noted that margins are set based on the
       pPrintableW & pPrintableH. This is the printable area your default
       printer supports e.g. though the Physical page size is 8.5x11 but
       the printer can print only on an area 8x11 in this case after
       setting the above margin you can print in an area of 6x9.
       IMPORTANT!!! After setting the margin your starting point is always
       0,0. So all you calls to draw on the canvas should be in an area
       6x9. Any thing outside this area will be clipped. Also if you maintain
       your drawing area constant you can change the margin afterwards
       without changing you drawing routins.}
       ShowMargin := True;
       ShowGrid := True;
       Orientation := poPortrait;    {poPortrait poLandscape}
       NewJob;
           with Canvases[CurrentPage] do begin
              w := pPrintableW*1440-Margins.Left-Margins.Right;
          {Text Alignements}

              font.name := 'Times New Roman';
              font.size := 14;
              font.style := [fsBold];
              SetTextAlign(Handle,TA_LEFT);   {Windows API}
              Textout(half,half,'Left Aligned');{Delphi}
              SetTextAlign(Handle,TA_CENTER);
              Textout(round(w/2),half,'RK-Printer');{Center of Page}
              SetTextAlign(Handle,TA_RIGHT);
              Textout(w,half,'1234.50');
              h := TxtHeight('X');
              TextOut(w,half+h,'100.00');
              SetTextAlign(Handle,TA_LEFT);

              font.size := 12;
              font.style := []; {Normal}
              h := TxtHeight('X');
              {Observe that the y axis is the same in following calls but due
               to different alignments placement of text is different. The line
               is drawn to see the difference, see the placement of 'y'}

              SetTextAlign(Handle,TA_TOP); {The default}
              TextOut(half,inch,'TA_TOP');

              SetTextAlign(Handle,TA_BOTTOM);
              TextOut(half+(TxtWidth('TA_TOP ')),inch,'TA_BOTTOMy');

              SetTextAlign(Handle,TA_BASELINE);
              TextOut(half+(TxtWidth('TA TOP TA_BOTTOMy  ')),inch,'TA_BASELINEy');
              moveto(0,inch);
              LineTo(w,inch);
              SetTextAlign(Handle,TA_LEFT);

          {Tabulation}
              {Put the cursor on the Function name and press F1 to know more about it}
              s := #9+'First'+#9+'Second'+#9+'Third'+#9+'Fourth';
              tabs[1] := inch; tabs[2] := 2880; tabs[3] := 3960; tabs[4] := 5400;
              TabbedTextOut(Handle,0,2880,@s[1],Length(s),4,tabs,0);
              TabbedTextOut(Handle,0,2880+h,@s[1],Length(s),4,tabs,0);
              TabbedTextOut(Handle,0,2880+h*2,@s[1],Length(s),4,tabs,360);
              TabbedTextOut(Handle,0,2880+h*3,@s[1],Length(s),4,tabs,360);
               {now an offset of 360 is added to the tabs}
          {Text Drawing}
              myRect := Rect(360,4320,360+inch+130,4320+h);
              with myRect do Rectangle(Left,Top,Right,Bottom);
              s := 'This can be clipped';
              ExtTextOut(Handle,360,4320,ETO_CLIPPED,@myRect,@s[1],Length(s),NIL);
              oldColor := GetBkColor(Handle);
              SetBkColor(Handle,ColorToRGB(clAqua));
              myRect := Rect(2160,4320,2160+inch+130,4320+h);
              ExtTextOut(Handle,2160,4320,ETO_OPAQUE,@myRect,@s[1],Length(s),NIL);
              tabs[1]:=230; tabs[2]:=240; tabs[3]:=350; tabs[4]:=460;{Character spacing}
              myRect := Rect(4320,4320,4320+inch+130,4320+h);
              s := 'Hello';
              ExtTextOut(Handle,4320,4320,ETO_OPAQUE+ETO_CLIPPED,@myRect,@s[1],Length(s),@tabs[1]);
              SetBkColor(Handle,oldColor);

              myRect := Rect(360,5040,1800,6840);
              with myRect do Rectangle(Left,Top,Right,Bottom);
              s := 'RKPrinter';
              {IMPORTANT!!! The following code is a work around but AVOID using
               DrawText function as we are using Metafiles and it NEVER
               returns any value like a true GDI or Canvas. That is the reason
               TxtWidth and TxtHeight Methods are provided to get the text width
               and height. DrawText uses internally Textwidth & Height function
               to break a line and wordwrap. Press the DrawText Button to see how
               it functions properly when a true Canvas is used and how it fails
               here.}
              SetTextAlign(Handle,TA_TOP+TA_RIGHT);

              saveDC(Handle);  {Saving the device Context}
                  DrawText(Handle,@s[1],Length(s),myRect,
                       DT_VCENTER+DT_RIGHT+DT_SINGLELINE+DT_NOCLIP);
                  SetTextAlign(Handle,TA_TOP+TA_LEFT);
                  {The following code will fail}
                  myRect := Rect(1880,5040,1880+1440,6840);
                  with myRect do Rectangle(Left,Top,Right,Bottom);
                  s := 'First Second Third Fourth'+#0;
                  DrawText(Handle,@s[1],-1,myRect,DT_WORDBREAK);
              RestoreDC(Handle,-1); {Restoring the device Context. Always
                use SaveDC RestoreDC in Pair the -1 parameter above will
                restore the previouse saved DC. I had to use this here as
                the following code for Bitmap doesnot function without it.
                Can anyone tell me why? Is it becuase Drawtext fails and
                does something funny with the DC?? }
                SetTextAlign(Handle,TA_TOP+TA_LEFT);
          {Bitmaps}

              b := TBitmap.Create;  {Change the line below for other BMP}
              b.loadFromFile('insoft.bmp');
              h := (w-b.width*10) div 2; {Center the Graphic 20X size}
              StretchDraw(rect(h,7200,b.width*10+h,b.height*10+7200),b);
              b.free;
              {Delphi provides the width & height of a bitmap, depending on
               how you want to place it you can change the above line to
               place you BMP on the page.}

           end;
       DonePrinting;
       Preview;
   end;
   R.free;
end;

procedure TForm1.Button3Click(Sender: TObject);
var
   myRect : TRect;
   s: String;
begin
   (*This gives proper result in {1} it breaks the word, in {2} it will
    understand newline and breakit at #13. Comment {1} and remove the
    comment from {2} to see different effect*)
   with Canvas do begin
       myRect := Rect(10,70,100,192);
       Rectangle(10,70,100,192);
       s := 'First Second Third Fourth'+#0; {1}
       {s := 'First'+#13+'Second'+#13+'Third'+#13+'Fourth'+#0;}  {2}
       DrawText(Handle,@s[1],-1,myRect,DT_WORDBREAK); {1}
       {DrawText(Handle,@s[1],-1,myRect,DT_LEFT);}               {2}
   end;
end;

end.
